# HighNoon by David Bowland
# ./addons/eventscripts/highnoon/highnoon.py

# >>> To configure this addon please see highnoon.cfg (created when the addon is first loaded) <<<

"""
Changes the brightness of each map based on the time the map started. Can simulate multiple 24-hours cycles in a single day.
Requires only ES 2.0.0.250c+
"""


from __future__ import division

import cfglib
import cmdlib
import es
import gamethread
import os.path
import time

import psyco
psyco.full()


info = es.AddonInfo()
info.name     = 'HighNoon'
info.version  = '2'
info.basename = 'highnoon'
info.url      = 'http://addons.eventscripts.com/addons/view/' + info.basename
info.author   = 'SuperDave'

###

config = cfglib.AddonCFG(os.path.join(es.getAddonPath(info.basename), info.basename + '.cfg'))

config.text(info.name + ' release %s options' % info.version)
config.text('./addons/eventscripts/%(basename)s/%(basename)s.cfg' % {'basename': info.basename})
config.text(info.url)
config.text('')
config.text('Load this addon with: es_load ' + info.basename)
config.text('\n')
config.text('To configure language settings for %s please see %s_languages.ini' % (info.name, info.basename))
config.text('\n')

config.text('***** General options *****')
cvar_midnight       = config.cvar('highnoon_midnight',       '00:00', 'Server time that represents midnight of the first cycle in the format "H:M" where H is 0-23 and M is 0-59')
cvar_cycles         = config.cvar('highnoon_cycles',         8,       'Number of day cycles in a 24 hour period')
cvar_brightness_min = config.cvar('highnoon_brightness_min', 1,       'Minimum brightness from 0 to 25 inclusive')
cvar_brightness_max = config.cvar('highnoon_brightness_max', 25,      'Maximum brightness from 0 to 25 inclusive')
config.text('\n')

config.text('***** Skybox options *****')
config.text('')
config.text('Use the following server command to add skyboxes to HighNoon:')
config.text('')
config.text('highnoon_add <skybox>')
config.text('\n')

config.text('Place your highnoon_add commands below from darkest to lightest:')
config.command('highnoon_add')
config.text('\n')

config.text('Uncomment the following default options if you wish to use them:')
config.text('')
config.text('highnoon_add sky_borealis01')
config.text('highnoon_add sky_day01_09')
config.text('highnoon_add sky_day02_09')
config.text('highnoon_add sky_day03_06')
config.text('highnoon_add hav')
config.text('highnoon_add sky_day03_03')
config.text('highnoon_add jungle')
config.text('highnoon_add de_cobble')
config.text('highnoon_add sky_day01_08')
config.text('highnoon_add sky_day01_07')
config.text('highnoon_add sky_day01_06')
config.text('highnoon_add sky_day03_04')
config.text('highnoon_add sky_day03_02')
config.text('highnoon_add sky_day03_01')
config.text('highnoon_add sky_day03_05')
config.text('highnoon_add sky_day01_01')
config.text('highnoon_add de_piranesi')
config.text('highnoon_add italy')
config.text('highnoon_add sky_day02_07')
config.text('highnoon_add sky_day02_06')
config.text('highnoon_add office')
config.text('highnoon_add sky_day01_05')
config.text('highnoon_add sky_day02_05')
config.text('highnoon_add sky_day01_04')
config.text('highnoon_add sky_day02_04')
config.text('highnoon_add sky_day02_03')
config.text('highnoon_add sky_day02_02')
config.text('highnoon_add sky_day02_01')
config.text('highnoon_add train')
config.text('highnoon_add tides')
config.text('highnoon_add cx')
config.text('highnoon_add sky_dust')
config.text('\n')

config.text('Skyboxes can be removed with the following server command:')
config.text('')
config.text('highnoon_remove <skybox>')
config.text('')
config.text('Due to the fact skyboxes can be added and removed above, this command will not be used by most users.')
config.text('\n')

config.text('***** Documentation *****')
config.text('')
config.text('The percent of the day cycle that has passed can be set for the next map using the highnoon_override cvar,')
config.text('such that 0 is midnight, 50 is noon, and 100 is midnight again.')
config.text('Said variable resets itself to -1 (no change) every map.')
config.text('')
config.text('Using highnoon_override you can set a static time for certain maps by setting its value in a .cfg file you create')
config.text('at: ./maps/cfg/<mapname>.cfg')
config.text('Such that to override the real time to midnight on de_dust you would put the following (uncommented) in: ./maps/cfg/de_dust.cfg')
config.text('highnoon_override 0')
config.text('')
config.text('You can unload HighNoon on specific maps by putting the following (uncommented) in your ./cfg/server.cfg:')
config.text('es_load highnoon')
config.text('You can then create a .cfg file for the maps on which you want HighNoon to be unloaded and unload it,')
config.text('using the cfg format: ./maps/cfg/<mapname>.cfg')
config.text('Such that to unload HighNoon on the map de_dust you would put the following (uncommented) in ./maps/cfg/de_dust.cfg')
config.text('es_unload highnoon')

config.write()


# This variable is for use by other addons so it shouldn't be included in the cfg
cvar_override = es.ServerVar('highnoon_override', -1, 'Overrides the percent of the cycle that has passed for the next map')

###

class HighNoonManager(object):
   """ Handles map brightness and skyboxes """
   sv_skyname = es.ServerVar('sv_skyname')

   def __init__(self):
      self.skyboxes = []
      cmdlib.registerServerCommand('highnoon_add', self.addSkyBox, 'highnoon_add <skybox>\nServer command to add skyboxes to HighNoon')
      cmdlib.registerServerCommand('highnoon_remove', self.removeSkyBox, 'highnoon_remove <skybox>\nServer command to remove skyboxes from HighNoon')

   """ Brightness functions """

   def setBrightness(self):
      """ Sets the map's brightness by setting the skybox and light style """
      self.cancelDelayed()

      # Determines the length of a day cycle as well as the midnight starting point
      cycle = (24 * 60) / cvar_cycles
      cyclestart = map(int, str(cvar_midnight).split(':'))
      midnight = cyclestart[0] + cyclestart[1] * 60

      # Determins the cycle percent based on the override variable or the current time
      if 0 <= cvar_override <= 100:
         now = cycle * cvar_override / 100
      else:
         localtime = time.localtime()
         now = abs(localtime.tm_hour * 60 + localtime.tm_min - midnight * 60) % cycle
      cvar_override.set(-1)

      # Sets the brightness of the map
      brightness_range = range(cvar_brightness_min, cvar_brightness_max) + range(cvar_brightness_max, cvar_brightness_min, -1)
      lightstyle = chr(97 + brightness_range[int(now / (cycle / len(brightness_range)))])
      es.lightstyle(0, lightstyle)

      # Sets the map's skybox
      skybox_range = self.skyboxes + list(reversed(self.skyboxes))[1:~0]
      if skybox_range:
         self.sv_skyname.set(skybox_range[int(now / (float(cycle) / len(skybox_range)))])

   """ Server command functions """

   def addSkyBox(self, args):
      """
      highnoon_add <skybox>
      """
      if len(args) == 1:
         self.skyboxes.append(args[0])
      else:
         es.dbgmsg(0, 'Syntax: highnoon_add <skybox>')

   def removeSkyBox(self, args):
      """
      highnoon_remove <skybox>
      """
      if len(args) == 1:
         name = args[0]
         if name in self.skyboxes:
            self.skyboxes.remove(name)
      else:
         es.dbgmsg(0, 'Syntax: highnoon_remove <skybox>')

   """ Static functions """

   @staticmethod
   def cancelDelayed():
      """
      Cancels the delay, however brief, between map start and setting the brightness
      This functionality is primarily for servers that unload HighNoon in server.cfg
      """
      gamethread.cancelDelayed('highnoon')

   @staticmethod
   def unregisterCommands():
      """ Unregisters the server commands """
      cmdlib.unregisterServerCommand('highnoon_add')
      cmdlib.unregisterServerCommand('highnoon_remove')

highnoon = HighNoonManager()

###

def load():
   config.execute()


def es_map_start(event_var):
   # Delays changing the map settings so the addon can be unloaded in a cfg that executes before the map starts
   gamethread.delayedname(0.1, 'highnoon', highnoon.setBrightness)


def unload():
   highnoon.cancelDelayed()
   highnoon.unregisterCommands()

   cvar_override.set(0)
   for cvar in config.getCvars():
      es.ServerVar(cvar).set(0)